@@ -3,7 +3,6 @@
 #include "textpathitem.h"
 
 #define CHAR_RATIO     0.55
-#define MAX_ANGLE      30
 #define PADDING        2
 
 static void swap(const QLineF &line, QPointF *p1, QPointF *p2)
@@ -174,23 +173,25 @@ static QList<QPolygonF> polyLines(const QPainterPath &path, const QRectF &rect)
 	bool lastIn = rect.contains(path.elementAt(0));
 
 	for (int i = 1; i < path.elementCount(); i++) {
-		if (rect.contains(path.elementAt(i))) {
+		QPainterPath::Element e(path.elementAt(i));
+
+		if (!e.isMoveTo() && rect.contains(e)) {
 			if (lastIn) {
 				if (line.isEmpty())
 					line.append(path.elementAt(i-1));
-				line.append(path.elementAt(i));
+				line.append(e);
 			} else {
 				QPointF p;
-				QLineF l(path.elementAt(i-1), path.elementAt(i));
+				QLineF l(path.elementAt(i-1), e);
 
 				if (intersection(l, rect, &p))
 					line.append(p);
-				line.append(path.elementAt(i));
+				line.append(e);
 			}
 
 			lastIn = true;
-		} else {
-			QLineF l(path.elementAt(i-1), path.elementAt(i));
+		} else if (!e.isMoveTo()) {
+			QLineF l(path.elementAt(i-1), e);
 
 			if (lastIn) {
 				QPointF p;
@@ -211,6 +212,14 @@ static QList<QPolygonF> polyLines(const QPainterPath &path, const QRectF &rect)
 			}
 
 			lastIn = false;
+		} else {
+			if (lastIn)
+				line.append(path.elementAt(i-1));
+
+			lines.append(line);
+			line.clear();
+
+			lastIn = rect.contains(e);
 		}
 	}
 
@@ -235,7 +244,7 @@ static qreal diff(qreal a1, qreal a2)
 
 template<class T>
 static QPainterPath textPath(const T &path, qreal textWidth, qreal charWidth,
-  const QRectF &tileRect)
+  const QRectF &tileRect, qreal maxAngle)
 {
 	if (path.isEmpty())
 		return QPainterPath();
@@ -257,7 +266,7 @@ static QPainterPath textPath(const T &path, qreal textWidth, qreal charWidth,
 					return subpath(pl, last, j - 1, length - textWidth);
 				last = j;
 				length = 0;
-			} else if (j > 1 && diff(angle, a) > MAX_ANGLE) {
+			} else if (j > 1 && diff(angle, a) > maxAngle) {
 				if (length > textWidth)
 					return subpath(pl, last, j - 1, length - textWidth);
 				last = j - 1;
@@ -276,7 +285,7 @@ static QPainterPath textPath(const T &path, qreal textWidth, qreal charWidth,
 }
 
 template<class T>
-void TextPathItem::init(const T &line, const QRect &tileRect)
+void TextPathItem::init(const T &line, const QRect &tileRect, qreal maxAngle)
 {
 	qreal cw, mw, textWidth;
 	bool label = _text && _font;
@@ -298,14 +307,17 @@ void TextPathItem::init(const T &line, const QRect &tileRect)
 		textWidth = _img->width() / _img->devicePixelRatioF();
 	}
 
-	_path = textPath(line, textWidth, cw, tileRect.adjusted(mw, mw, -mw, -mw));
+	_path = textPath(line, textWidth, cw, tileRect.adjusted(mw, mw, -mw, -mw),
+					 maxAngle);
 	if (_path.isEmpty())
 		return;
-
 	if (reverse(_path)) {
 		_path = _path.toReversed();
 		_reverse = true;
 	}
+#if QT_VERSION >= QT_VERSION_CHECK(6, 10, 0)
+	_path.setCachingEnabled(true);
+#endif // QT 6.10
 
 	QPainterPathStroker s;
 	s.setWidth(mw * 2);
@@ -316,20 +328,20 @@ void TextPathItem::init(const T &line, const QRect &tileRect)
 
 TextPathItem::TextPathItem(const QPolygonF &line, const QString *label,
   const QRect &tileRect, const QFont *font, const QColor *color,
-  const QColor *haloColor, const QImage *img, bool rotate)
+  const QColor *haloColor, const QImage *img, bool rotate, qreal maxAngle)
   : TextItem(label), _font(font), _color(color), _haloColor(haloColor),
   _img(img), _rotate(rotate), _reverse(false)
 {
-	init(line, tileRect);
+	init(line, tileRect, maxAngle);
 }
 
 TextPathItem::TextPathItem(const QPainterPath &line, const QString *label,
   const QRect &tileRect, const QFont *font, const QColor *color,
-  const QColor *haloColor, const QImage *img, bool rotate)
+  const QColor *haloColor, const QImage *img, bool rotate, qreal maxAngle)
   : TextItem(label), _font(font), _color(color), _haloColor(haloColor),
 	_img(img), _rotate(rotate), _reverse(false)
 {
-	init(line, tileRect);
+	init(line, tileRect, maxAngle);
 }
 
 void TextPathItem::paint(QPainter *painter) const
